@@ -26,7 +26,6 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
-from gi.repository import Atspi, Atk
 
 from . import braille
 from . import debug
@@ -158,38 +157,25 @@ class BrailleGenerator(generator.Generator):
 
         elif verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
            and not args.get('readingRow', False) and role not in doNotPresent:
-            result.append(self.getLocalizedRoleName(obj, role))
+            result.append(self.getLocalizedRoleName(obj, **args))
         return result
 
-    def getLocalizedRoleName(self, obj, role=None):
+    def getLocalizedRoleName(self, obj, **args):
         """Returns the localized name of the given Accessible object; the name
         is suitable to be brailled.
 
         Arguments:
         - obj: an Accessible object
-        - role: an optional pyatspi role to use instead
         """
 
         if _settingsManager.getSetting('brailleRolenameStyle') \
                 == settings.BRAILLE_ROLENAME_STYLE_SHORT:
-            objRole = role or obj.getRole()
-            rv = shortRoleNames.get(objRole)
+            role = args.get('role', obj.getRole())
+            rv = shortRoleNames.get(role)
             if rv:
                 return rv
 
-        if not isinstance(role, pyatspi.Role):
-            try:
-                return obj.getLocalizedRoleName()
-            except:
-                return ''
-
-        if not role:
-            return ''
-
-        nonlocalized = Atspi.role_get_name(role)
-        atkRole = Atk.role_for_name(nonlocalized)
-
-        return Atk.role_get_localized_name(atkRole)
+        return super().getLocalizedRoleName(obj, **args)
 
     def _generateUnrelatedLabels(self, obj, **args):
         result = []
@@ -327,7 +313,7 @@ class BrailleGenerator(generator.Generator):
         if 'Selection' in pyatspi.listInterfaces(obj):
             items = self._script.utilities.selectedChildren(obj)
         else:
-            items = items or [self._script.utilities.focusedChild(obj)]
+            items = [self._script.utilities.focusedChild(obj)]
         if not (items and items[0]):
             return result
 
@@ -355,27 +341,43 @@ class BrailleGenerator(generator.Generator):
 
     def _generateProgressBarIndex(self, obj, **args):
         if not args.get('isProgressBarUpdate') \
-           or not _settingsManager.getSetting('brailleProgressBarUpdates'):
+           or not self._shouldPresentProgressBarUpdate(obj, **args):
             return []
 
-        acc, updateTime, updateValue = self._script.utilities.getMostRecentProgressBarUpdate()
+        acc, updateTime, updateValue = self._getMostRecentProgressBarUpdate()
         if acc != obj:
-            number, count = self._script.utilities.getProgressBarNumberAndCount(obj)
+            number, count = self.getProgressBarNumberAndCount(obj)
             return ['%s' % number]
 
         return []
 
     def _generateProgressBarValue(self, obj, **args):
         if args.get('isProgressBarUpdate') \
-           and not _settingsManager.getSetting('brailleProgressBarUpdates'):
+           and not self._shouldPresentProgressBarUpdate(obj, **args):
             return []
 
+        return self._generatePercentage(obj, **args)
+
+    def _generatePercentage(self, obj, **args):
         percent = self._script.utilities.getValueAsPercent(obj)
         if percent is not None:
             return ['%s%%' % percent]
 
         return []
 
+    def _getProgressBarUpdateInterval(self):
+        interval = _settingsManager.getSetting('progressBarBrailleInterval')
+        if interval is None:
+            return super()._getProgressBarUpdateInterval()
+
+        return int(interval)
+
+    def _shouldPresentProgressBarUpdate(self, obj, **args):
+        if not _settingsManager.getSetting('brailleProgressBarUpdates'):
+            return False
+
+        return super()._shouldPresentProgressBarUpdate(obj, **args)
+
     #####################################################################
     #                                                                   #
     # Unfortunate hacks.                                                #
