@@ -69,6 +69,9 @@ keymap = (
     ("t", defaultModifierMask, ORCA_MODIFIER_MASK,
     "presentDateHandler", 2),
 
+    ("Up", defaultModifierMask, ORCA_SHIFT_MODIFIER_MASK,
+    "whereAmISelectionHandler"),
+
     #####################################################################
     #                                                                   #
     #  Bookmark key bindings                                            #
@@ -118,22 +121,6 @@ keymap = (
     ("6", defaultModifierMask, ORCA_MODIFIER_MASK,
     "goToBookmark"),
 
-    # key binding for WhereAmI information with respect to root acc
-
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-    ("", defaultModifierMask, NO_MODIFIER_MASK,
-    "bookmarkCurrentWhereAmI"),
-
-
     ("BackSpace", defaultModifierMask, ORCA_MODIFIER_MASK,
     "bypassNextCommandHandler"),
 
@@ -189,6 +176,9 @@ keymap = (
     ("", defaultModifierMask, NO_MODIFIER_MASK,
     "cycleKeyEchoHandler"),
 
+    ("", defaultModifierMask, NO_MODIFIER_MASK,
+    "changeNumberStyleHandler"),
+
     ("", defaultModifierMask, NO_MODIFIER_MASK,
     "repeatLastNotificationMessageHandler"),
 
@@ -206,4 +196,7 @@ keymap = (
 
     ("", defaultModifierMask, NO_MODIFIER_MASK,
     "shutdownHandler"),
+
+    ("", defaultModifierMask, NO_MODIFIER_MASK,
+    "whereAmILinkHandler"),
 )
